/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */
package cz.xelfi.quoridor;

/**
 * Represents position of a fence on a {@link Board}. The fence can be oriented 
 * either in
 * {@link Orientation#HORIZONTAL} or {@link Orientation#VERTICAL} direction.
 * The faces postions are numbered 1-8 and A-H according to following
 * graph: <pre>
            H   G   F   E   D   C   B   A
            |   |   |   |   |   |   |   |
        +-----------------------------------+
        |                                   |
     1--|   +   +   +   +   +   +   +   +   |--1
        |                                   |
     2--|   +   +   +   +   +   +   +   +   |--2
        |                                   |
     3--|   +   +   +   +   +   +   +   +   |--3
        |                                   |
     4--|   +   +   +   +   +   +   +   +   |--4
[E]     |                                   |     [W]
     5--|   +   +   +   +   +   +   +   +   |--5
        |                                   |
     6--|   +   +   +   +   +   +   +   +   |--6
        |                                   |
     7--|   +   +   +   +   +   +   +   +   |--7
        |                                   |
     8--|   +   +   +   +   +   +   +   +   |--8
        |                                   |
        +-----------------------------------+
            |   |   |   |   |   |   |   |
            H   G   F   E   D   C   B   A
     * </pre>
 *
 */
public final class Fence extends Object {

    private final int x;
    private final int y;
    private final Orientation o;

    Fence(int x, int y, Orientation o) {
        super();
        this.x = x;
        this.y = y;
        this.o = o;
    }

    /** The column of the center of the fence. The same
     * as used in {@link Move#fence(char, int, cz.xelfi.quoridor.Fence.Orientation)}.
     *
     * @return a letter between 'A' and 'H'
     */
    public char getColumn() {
        return (char) ('A' + (x - 1) / 2);
    }

    /** The row of the center of the fence. The same
     * as specified in {@link Move#fence(char, int, cz.xelfi.quoridor.Fence.Orientation)}.
     *
     * @return a number between 1 and 8
     */
    public int getRow() {
        return (y + 1) / 2;
    }

    /** Midle coordinate of the fence.
     * @return 1-15
     */
    int getX() {
        return x;
    }

    /** Midle coordinate of the fence.
     * @return 1-15
     */
    int getY() {
        return y;
    }

    /** The orientation of the fence.
     * 
     * @return HORIZONTAL or VERTICAL
     */
    public Orientation getOrientation() {
        return o;
    }

    @Override
    public String toString() {
        return "Fence[" + getColumn() + "," + getRow() + "," + o + "]";
    }

    @Override
    public int hashCode() {
        return 8 * x + 4 * y + 13 + o.hashCode();
    }

    @Override
    public boolean equals(Object o) {
        if (o instanceof Fence) {
            Fence f = (Fence) o;
            return x == f.x && y == f.y && getOrientation().equals(f.getOrientation());
        }
        return false;
    }

    /** The possible orientation of a {@link Fence}.
     * Either horizontal of vertical.
     */
    public static enum Orientation {
        HORIZONTAL, VERTICAL;

        private Orientation() {
        }
    }
}
